import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Servlet that updates a shared counter
 * Notice that the shared counter is stored
 * as the attribute of the servlet context.
 *
 * @author Daqing Hou
 */

public class CounterContext extends HttpServlet {


    public void doGet(HttpServletRequest request,
                      HttpServletResponse response)
        throws IOException, ServletException
    {
        response.setContentType("text/html");

        PrintWriter out = response.getWriter();
        out.println("<html>");
        out.println("<body>");
        out.println("<head>");

        out.println("<title> Counter  </title>");
        out.println("</head>");
        out.println("<body bgcolor=\"orange\">");

        out.println("<h3> Counter = ");

        ServletContext context = getServletContext();      
        out.println(readCounter(context));
        out.println("</h3>");

        out.println("<P>");
        out.print("<form action=\"CounterContext\" method=POST>");
        out.println("<input type=text size=20 name=increment>");
        out.println("<br>");
        out.println("<input type=submit>");
        out.println("</form>");

        out.print("<form action=\"CounterPersistence\" method=GET>");
        out.println("<input type=submit value=\"stop\">");
        out.println("</form>");

        out.println("</body>");
        out.println("</html>");
    }

    public void doPost(HttpServletRequest request,
                      HttpServletResponse response)
        throws IOException, ServletException
    {
        response.setContentType("text/html");

        PrintWriter out = response.getWriter();
        out.println("<html>");
        out.println("<body>");
        out.println("<head>");

        out.println("<title> Counter  </title>");
        out.println("</head>");
        out.println("<body bgcolor=\"orange\">");

        out.println("<h3> Counter = ");

        ServletContext context = getServletContext();      
        int val = readCounter(context);
        String increment = request.getParameter("increment");
        int inc = Integer.parseInt(increment);

        out.println(Integer.toString(val+inc));
        out.println("</h3>");

        writeCounter(val+inc, context);

        out.println("<P>");
        out.print("<form action=\"CounterContext\" method=POST>");
        out.println("<input type=text size=20 name=increment>");
        out.println("<br>");
        out.println("<input type=submit>");
        out.println("</form>");

        out.print("<form action=\"CounterPersistence\" method=GET>");
        out.println("<input type=submit value=\"stop\">");
        out.println("</form>");

        out.println("</body>");
        out.println("</html>");
    }

   private static void writeCounter(int counterVal, ServletContext context){
        InMemoryCounter counter = (InMemoryCounter)context.getAttribute("counter");
        counter.set(counterVal);
   }

   private static int readCounter(ServletContext context){
        InMemoryCounter counter = null;
        try {
        counter = (InMemoryCounter)context.getAttribute("counter");
        }
        catch (ClassCastException cce){
        counter = null;
        }

        if (counter==null){
        String path = context.getRealPath("/") + "WEB-INF/classes/Counter.data";
        counter = new InMemoryCounter(path);
        context.setAttribute("counter", counter);
        }
        return counter.val();
   }

   static int readCounter(String path){
        int i=0;

        char buf[] = new char[12];
        try {
        Reader reader = new FileReader(path);
        int j=-1, c;
        c=reader.read();
        while (Character.isDigit(c)){
           ++j;
           buf[j]=(char)c;
           c=reader.read();
        }
        String s = new String(buf, 0, j+1);
        i = Integer.parseInt(s);
        reader.close();
        }
        catch (FileNotFoundException fnfe) {
           i=-1;
        }
        catch (EOFException eofe) {
           i=-2;
        }
        catch (IOException ioe) {
           i=-3;
        }
        return i; 
   }
   static void writeCounter(int counter, String filePath){
       try {
       Writer writer = new FileWriter(filePath);
       writer.write(Integer.toString(counter));
       writer.close();
       }
       catch (IOException ioe){
       }
   }
}
